// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


/*
   Establece un manejador externo para controlar el teclado, recoge los parametros de la operacion
   por este y envia la peticion al servidor que devolvera el resultado de aplicar la operacion 
   sobre los parametros recogidos.

   El cliente de esta aplicacion es: service.p => Transporte: comm::Transport.
*/
#include <iostream>

#include <string.h>

#include <anna/core/core.hpp>
#include <anna/comm/comm.hpp>

#include <anna/test/Menu.hpp>
#include <anna/test/Request.hpp>
#include <anna/test/Response.hpp>

class MyCommunicator : public Communicator {
public:   
   MyCommunicator () : Communicator () {;}
   
private:
   test::Response a_response;   
   test::Request a_request;
   
   void eventReceiveMessage (ClientSocket&, const Message&) throw (RuntimeException);
   void eventUser (const char* id, const void* context) throw ();   
};

class RRKClient : public anna::comm::Application {
public:
   RRKClient ();
      
   Service* getService () const throw () { return a_service; }
   const test::Menu& getMenu () const throw () { return a_menu; }
   
private:
   MyCommunicator a_communicator;
   test::Menu a_menu;
   Service* a_service;

   void initialize () throw (RuntimeException);
   void run () throw (RuntimeException);    
};

using namespace std;
using namespace test;

int main (int argc, const char** argv)
{
   CommandLine& commandLine (CommandLine::instantiate ());
   RRKClient app;
   
   try {
      commandLine.initialize (argv, argc);
      commandLine.verify ();

      Logger::setLevel (Logger::Debug); 
      Logger::initialize ("kclient", new TraceWriter ("file.trace", 4096000));
 
      app.start ();
   }
   catch (Exception& ex) {
      cout << ex.asString () << endl;
   }
   
   return 0;
}

RRKClient::RRKClient () : 
   Application ("kclient", "RRKClient", "1.0.0"),
   a_menu (&a_communicator)
{
   CommandLine& commandLine (CommandLine::instantiate ());
      
   commandLine.add ("p", CommandLine::Argument::Mandatory, "Puertos en los que el servidor atiende respuestas.");
   commandLine.add ("a", CommandLine::Argument::Mandatory, "Puertos en los que hay servidores atendiendo peticiones");
   commandLine.add ("trace", CommandLine::Argument::Optional, "Nivel de trazas (debug,warning, error,...)");
}

void RRKClient::initialize () 
   throw (RuntimeException)
{
   CommandLine& cl (CommandLine::instantiate ());    

   if (cl.exists ("trace"))
      Logger::setLevel (Logger::asLevel (cl.getValue ("trace")));

   Network& network = Network::instantiate ();

   Tokenizer ports (cl.getValue ("p"), ",");
   int port;

   a_service = new comm::RoundRobinDelivery ("Service_Arithmetic", true);

   for (Tokenizer::const_iterator ii = ports.begin (), maxii = ports.end (); ii != maxii; ii ++) {
      port = atoi (Tokenizer::data (ii));
      a_service->attach (network.createServer (cl.getValue ("a"), port, true));
   }

   a_communicator.attach (a_service);
   a_communicator.attach (&a_menu);
}

void RRKClient::run ()
   throw (RuntimeException)
{   
   a_menu.paint ();
   a_communicator.accept ();
}

void MyCommunicator::eventReceiveMessage (ClientSocket&, const Message& message)
   throw (RuntimeException)
{
   LOGMETHOD (TraceMethod tm (Logger::Local7, "MyCommunicator", "eventReceiveMessage", ANNA_FILE_LOCATION));

   a_response.decode (message.getBody ());

   const Millisecond responseTime = anna::functions::millisecond () - a_response.initTime;
   
   cout << endl << "ResponseTime: " << responseTime << " ms" << endl;
   cout << "Resultado de la peticion: " << a_response.x << (char) a_response.op << a_response.y  << " = " << a_response.result << endl << endl;
   
   static_cast <RRKClient&> (anna::comm::functions::getApp ()).getMenu ().paint ();
}

//-----------------------------------------------------------------------------------------
// Cuando el Menu tiene disponibles todos los datos necesarios para la peticin se lo
// notifica al comunicador mediante un evento de usuario.
//-----------------------------------------------------------------------------------------
void MyCommunicator::eventUser (const char* id, const void* context) 
   throw ()
{
   LOGMETHOD (TraceMethod tm (Logger::Local7, "MyCommunicator", "eventUser", ANNA_FILE_LOCATION));

   if (anna_strcmp (id, Menu::EventData) == 0) {
      const Menu::Data* data (reinterpret_cast <const Menu::Data*>  (context));
      
      a_request.op = data->a_operation;
      a_request.x = data->a_op1;
      a_request.y = data->a_op2;
      a_request.initTime = anna::functions::millisecond ();
      
      Service* service = static_cast <RRKClient&> (anna::comm::functions::getApp ()).getService ();
      
      try {
         service->send (a_request);
      }
      catch (RuntimeException& ex) {
         ex.trace ();
      }
   } 
}

