# KAFKA actions

## consume_json

Consume until reception of json message.

Arguments: json[, auto_offset_reset, timeout, background, debug]
* json: json file that we want to wait for ('any' to skip matching).
* timeout: timeout for the operation in seconds.
* auto_offset_reset: auto offset reset (earliest, latest, etc.). By default: latest.
* background: "yes"|["no"]. Executes the step in background. By default is blocking, allowing to gather the result code for the step.
* debug: "yes"|["no"]. Python kafka library debug information.

Example:
```
- action: SMPC/consume_json
  arguments:
    json: expected.json
    timeout: 5
    auto_offset_reset: earliest
```

## produce_json

Produces a json message to the kafka message bus.

Arguments: json[, delay_ms, background, debug]
* json: json file that we want to produce.
* delay_ms: delay before producing in milliseconds.
* background: "yes"|["no"]. Executes the step in background. By default is blocking, allowing to gather the result code for the step.
* debug: "yes"|["no"]. Python kafka library debug information.

Example:
```
- action: SMPC/produce_json:
  arguments:
    json: produced.json
```

## admin

Administration actions.

Arguments: operation[, debug]
* operation: only 'clean_topic' implemented
* debug: "yes"|["no"]. Python kafka library debug information.

clean_topic: administration operation to clear message bus in case you want to consume reading earliest messages in the queue. By default, consumer offsets to latest message, but it won't see nothing if it is launched after production of the expected message. Depending on the way you define the test case, you will need clean or not the message bus.

* Example(1): given an 'SMPC' kafka node:

```
- action: SMPC/admin:
  arguments:
    operation: clean_topic

- action: AF/send_xml_to_entity
  arguments:
    xml: AAR.xml

- action: SPMC/consume_json
  arguments:
    json: expected.json
    auto_offset_reset: earliest
```

* Example(2): given an 'SMPC' kafka node:

```
- action: SPMC/consume_json
  arguments:
    json: expected.json
    background: "yes"

- action: AF/send_xml_to_entity
  arguments:
    xml: AAR.xml
```

* Example(3): given an 'SMPC' kafka node:

```
- action: SPMC/produce_json
  arguments:
    json: produced.json
    delay_ms: 200
    background: "yes"

- action: AF/wait_xml_from_entity
  arguments:
    xml: ASR.xml
```

