// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


// Standard
#include <iostream>
#include <fstream>

// STL
#include <string>

#include <anna/core/core.hpp>
#include <anna/xml/xml.hpp>
#include <anna/diameter/stack/Engine.hpp>
#include <anna/time/functions.hpp>
#include <anna/time/Date.hpp>

#include <anna/diameter/codec/Engine.hpp>
#include <anna/diameter/codec/tme/Message.hpp>


typedef anna::diameter::codec::tme::Message tmeMessage;
typedef anna::diameter::codec::tme::Avp tmeAvp;


void _exit(const std::string & msg) {
  std::cout << std::endl << msg << std::endl;
  exit(-1);
}


std::string exec_cmd(const char* cmd) {
  FILE* pipe = popen(cmd, "r");

  if(!pipe) return "";

  char buffer[128];
  std::string result = "";

  while(!feof(pipe)) {
    if(fgets(buffer, 128, pipe) != NULL)
      result += buffer;
  }

  pclose(pipe);
  return result;
}

int main(int argc, char** argv) {
  Logger::setLevel(Logger::Debug);
  Logger::initialize("tme", new TraceWriter("file.trace", 2048000));
  anna::time::functions::initialize(); // before application instantiation (it have a anna::time object)
  anna::time::functions::setControlPoint(); // start control point (application lifetime)
  anna::diameter::stack::Engine & stackEngine = anna::diameter::stack::Engine::instantiate();
  anna::diameter::stack::Dictionary *dictionary;
  std::string exec = argv[0];
  std::string param = argv[1] ? argv[1] : "";

  if(argc < 2) {
    std::string msg = "Usage: "; msg += exec; msg += " <xml directory>";
    msg += "\n     xml directory: contains the xml files needed, which are:";
    msg += "\n";
    msg += "\n                       avps_ietf.xml";
    msg += "\n                       avps_etsi.xml";
    msg += "\n                       avps_tgpp.xml";
    msg += "\n                       avps_tme.xml";
    msg += "\n                       commands_dccaOCS-CS_HuaweiNGIN_de-es.xml";
    msg += "\n";
    msg += "\n     For example:   "; msg += exec; msg += " ";
    std::string projectRoot = exec_cmd("git rev-parse --show-toplevel");

    if(projectRoot != "") {
      //projectRoot.resize(projectRoot.size()-1);
      projectRoot.erase(projectRoot.size() - 1);
      msg += projectRoot;
      msg += "/";
    }

    msg += "source/diameter/stack/setups";
    _exit(msg);
  }

  try {
    dictionary = stackEngine.createDictionary(0 /* general unique stack id */);
    dictionary->allowUpdates();
    dictionary->load(param + "/avps_ietf.xml");
    dictionary->load(param + "/avps_etsi.xml");
    dictionary->load(param + "/avps_tgpp.xml");
    dictionary->load(param + "/avps_tme.xml");
    dictionary->load(param + "/commands_dccaOCS-CS_HuaweiNGIN_de-es.xml");
    // Trace:
    LOGINFORMATION(Logger::information(stackEngine.asString(), ANNA_FILE_LOCATION));
    LOGDEBUG(Logger::debug(dictionary->asString(), ANNA_FILE_LOCATION));
  } catch(anna::RuntimeException &ex) {
    ex.trace();
    std::cout << ex.getText() << std::endl;
  }

  // Build CCA
  anna::diameter::codec::Engine *codecEngine = new anna::diameter::codec::Engine("TME Engine", dictionary);
  LOGDEBUG(Logger::debug(codecEngine->asString(), ANNA_FILE_LOCATION));
  tmeMessage cca(codecEngine);
  cca.setId("Credit-Control-Answer");
  tmeAvp *sid = (tmeAvp*)cca.addAvp("Session-Id");
  tmeAvp *ohst = (tmeAvp*)cca.addAvp("Origin-Host");
  tmeAvp *orlm = (tmeAvp*)cca.addAvp("Origin-Realm");
  sid->getUTF8String()->setValue("grump.example.com:33041;23432;893;0AF3B81");
  ohst->getDiameterIdentity()->setValue("c0-10-70-238-74-w5p1vepg1.vepc.ericsson.se");
  orlm->getDiameterIdentity()->setValue("vepc.ericsson.se");
  int inRange = 65535;
  int outRange = inRange + 1;
  //((tmeAvp*)cca.addAvp("CS-Result-Code"))->getUnsigned16()->setValue(inRange);
  ((tmeAvp*)cca.addAvp("CS-Result-Code"))->getUnsigned16()->setValue(outRange); // WILL BE 0 !!
  /*
     <command name="Credit-Control-Answer" code="272" type="Answer">
        <avprule id="Session-Id" type="Fixed"/>
        <avprule id="Result-Code" type="Mandatory"/>
        <avprule id="CS-Result-Code" type="Optional"/>
        <avprule id="CS-Dialogue-Handling" type="Optional"/>
        <avprule id="Origin-Host" type="Mandatory"/>
        <avprule id="Origin-Realm" type="Mandatory"/>
        <avprule id="Auth-Application-Id" type="Mandatory"/>
        <avprule id="CC-Request-Type" type="Mandatory"/>
        <avprule id="CC-Request-Number" type="Mandatory"/>
        <avprule id="Granted-Service-Unit" type="Optional"/>
        <avprule id="Cost-Information" type="Optional"/>
        <avprule id="Low-Balance-Indication" type="Optional"/>
        <avprule id="Remaining-Balance" type="Optional"/>
        <avprule id="Credit-Control-Failure-Handling" type="Optional"/>
        <avprule id="CC-Correlation-Id" type="Optional"/>
        <avprule id="Redirect-Host" type="Optional" qual="*"/>
        <avprule id="Redirect-Host-Usage" type="Optional"/>
        <avprule id="Redirect-Max-Cache-Time" type="Optional"/>
        <avprule id="Proxy-Info" type="Optional" qual="*"/>
        <avprule id="Route-Record" type="Optional" qual="*"/>
        <avprule id="Failed-AVP" type="Optional" qual="*"/>
        <avprule id="Service-Information" type="Optional"/>
        <avprule id="AVP" type="Optional" qual="*"/>
     </command>
  */
  LOGDEBUG(Logger::debug(cca.asXMLString(), ANNA_FILE_LOCATION));
  _exit("Open 'file.trace' in order to see traces");
}

