// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_comm_ServerAllocator_hpp
#define anna_comm_ServerAllocator_hpp

#include <string>

namespace anna {

namespace comm {

class Host;
class TransportFactory;
class Server;

/**
   Instanciador de anna::comm::Server se usa para poder instanciar distintos tipos
   de anna::comm::Server desde la clase anna::comm::Host.

   \see Host::createServer
   \see Server
*/
class ServerAllocator {
public:
  /**
     Constructor. Recoge los parametros con los que crear el servidor.
     \param name Nombre logico del servidor.
     \param host Instancia de la maquina sobre la que esta atendiento peticiones.
     \param remotePort Puerto sobre el que atiende peticiones.
     \param autoRecovery Indica si en caso de caida se debe intentar la recuperacion
     automatica de la conexion.
     \param transportFactory Factoria de protocolos de transporte usada por los ClientSocket asociados a este
     proceso servidor.
     \param ignoreIncomingMessages Indicador de ignorar mensajes entrantes.
  */
  ServerAllocator(const std::string& name, const Host& host, const int remotePort, const bool autoRecovery, TransportFactory* transportFactory = NULL, const bool ignoreIncomingMessages = false) :
    a_name(name),
    a_host(host),
    a_remotePort(remotePort),
    a_autoRecovery(autoRecovery),
    a_transportFactory(transportFactory),
    a_ignoreIncomingMessages(ignoreIncomingMessages)
  {;}

  /**
     Constructor copia.
     \param other Instanciador del que copiar.
  */
  ServerAllocator(const ServerAllocator& other) :
    a_name(other.a_name),
    a_host(other.a_host),
    a_remotePort(other.a_remotePort),
    a_autoRecovery(other.a_autoRecovery),
    a_transportFactory(other.a_transportFactory),
    a_ignoreIncomingMessages(other.a_ignoreIncomingMessages)
  {;}

  /**
   * Destructor
   */
  virtual ~ServerAllocator() {;}

  /**
     Devuelve el nombre logico del servidor.
     \return el nombre logico del servidor.
  */
  const std::string& getName() const throw() { return a_name; }

  /**
     Devuelve el puerto sobre el que atiende peticiones.
     \return el puerto sobre el que atiende peticiones.
  */
  int getRemotePort() const throw() { return a_remotePort; }

  /**
     Indica si se ha indicado que en caso de caida se debe intentar la recuperacion
     automatica de la conexion.
     \return \em true si se ha indicado que en caso de caida se debe intentar la recuperacion
     automatica de la conexion o \em false en caso contrario.
  */
  bool autoRecovery() const throw() { return a_autoRecovery; }

  /**
   * Devuelve \em true si el indicador que ignora los mensajes entrantes est activo, o \em false en otro caso.
   * \return \em true si el indicador que ignora los mensajes entrantes est activo, o \em false en otro caso.
   */
  bool getIgnoreIncomingMessages() const throw() { return a_ignoreIncomingMessages; }

  /**
     Crea una instancia particular de anna::comm::Server usando los parametros establecidos
     en el constructor.
     \return una instancia particular de anna::comm::Server.
  */
  virtual Server* apply() const throw();

protected:
  const std::string a_name;
  const Host& a_host;
  const int a_remotePort;
  bool a_autoRecovery;
  TransportFactory* a_transportFactory;
  const bool a_ignoreIncomingMessages;
};

}

}

#endif
