// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_core_util_Component_hpp
#define anna_core_util_Component_hpp



#include <anna/core/RuntimeException.hpp>
#include <anna/core/mt/Mutex.hpp>

namespace anna {
namespace xml {
class Node;
}
}

namespace anna {

/**
   Component parent class

   It must be only one instance for each component, but we can't inherit them from anna::Singleton because the
   programmer must have the posibility to re-implement the component.

   \code
      Class* object = anna::functions::component <Class> (FILE_LOCATION);

      ..... object use ....
   \endcode

   If component 'Class' hasn't been registered, #anna::functions::component will launch an exception.
*/
class Component : public anna::Mutex {
public:
  /**
     Destructor.
  */
  virtual ~Component();

  /**
     Gets the class name configured at constructor.
     \return The class name configured at constructor.
  */
  const char* getClassName() const throw() { return a_className.c_str(); }

  /**
  * Class string representation
  *
  * @return String with class content
  */
  virtual std::string asString(void) const throw();

  /**
     Class XML representation.
     \param parent XML node over which we will put instance information.
     \return XML documentcon with class content.
  */
  virtual anna::xml::Node* asXML(anna::xml::Node* parent) const throw();


protected:

  const std::string a_className;

  /**
     Contructor.
     @param className Logical name for tha class.
  */
  explicit Component(const char* className);

  Component (const Component& other);
};

}

#endif

