// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_http_http_hpp
#define anna_http_http_hpp

namespace anna {

/**
   Proporciona los componentes necesarios para la comunicacion entre procesos usando como
   capa de transporte el protocolo HTTP 1.1 definido en el RFC 2616.

   A continacion mostramos la forma en la que se podria definir un ServerSocket que atiende peticiones y/o respuesta
   sobre el protocolo HTTP.

   \code
   void MyHTTPServer::initialize ()
      throw (RuntimeException)
   {
      CommandLine& cl (CommandLine::instantiate ());

      int port = cl.getIntegerValue ("p");
      const comm::Device* device = Network::instantiate ().find (Device::asAddress (cl.getValue ("a")));

      a_serverSocket = new ServerSocket (INetAddress (device, port), cl.exists ("r"), &anna::http::Transport::getFactory ());
   }
   \endcode

   El siguiente ejemplo muestra como podriamos definir un cliente HTTP que realiza las peticiones sobre un servidor
   HTTP:

   \code
   void MyHTTPClient::initialize ()
      throw (RuntimeException)
   {
      CommandLine& cl (CommandLine::instantiate ());

      Network& network = Network::instantiate ();

      Host* host = network.find ("host000");
      host->assign (network.find (Device::asAddress (cl.getValue ("a"))));
      a_server = host->createServer ("http_server", cl.getIntegerValue ("p"), true, &anna::http::Transport::getFactory ());
   }
   \endcode

   El ejecutable debera enlazarse con las librerias:
      \li libanna.core.a
      \li libanna.xml.a
      \li libanna.app.a
      \li libanna.comm.a
      \li libanna.http.a

   El <b>Packet Header</b> es anna.http.h
*/
namespace http {
}

}

#include <anna.http.functions.h>
#include <anna.http.Handler.h>
#include <anna.http.Header.h>
#include <anna.http.MessageFactory.h>
#include <anna.http.Message.h>
#include <anna.http.Method.h>
#include <anna.http.Request.h>
#include <anna.http.Response.h>
#include <anna.http.Transport.h>

using namespace anna::http;

#endif

