// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/core/tracing/Logger.hpp>
#include <anna/core/functions.hpp>
#include <anna/core/mt/Guard.hpp>

#include <anna/xml/Node.hpp>
#include <anna/xml/Attribute.hpp>

#include <anna/comm/ClientSocket.hpp>
#include <anna/comm/Communicator.hpp>
#include <anna/comm/Device.hpp>
#include <anna/comm/Receiver.hpp>

#include <anna/comm/handler/ClientSocket.hpp>

using namespace std;
using namespace anna;

void comm::handler::ClientSocket::initialize()
throw(RuntimeException) {
  if(a_clientSocket == NULL) {
    string msg(asString());
    msg += " | comm::ClientSocket was not established";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  if(a_clientSocket->isConnected() == false)
    a_clientSocket->connect();

  setfd(a_clientSocket->getfd());
}

/*
 * Se invoca desde Communicator::detach.
 *
 * [Tx] -> Communicator
 */
void comm::handler::ClientSocket::finalize()
throw() {
  if(a_clientSocket == NULL)
    return;

  Receiver* receiver;

  if((receiver = a_clientSocket->getReceiver()) != NULL)
    receiver->eventBreakConnection(*a_clientSocket);

  a_clientSocket->close();
  a_communicator->eventBreakConnection(*a_clientSocket);
  a_clientSocket = NULL;
}

void comm::handler::ClientSocket::clone()
throw(RuntimeException) {
  if(a_clientSocket->isConnected() == true)
    a_clientSocket->close();

  a_clientSocket->connect();
  const int fd = a_clientSocket->getfd();

  if(fd == -1) {
    string msg("comm::handler::ClientSocket::clone | ");
    msg += asString();
    msg += " | Cloning error";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  setfd(fd);
}

string comm::handler::ClientSocket::asString() const
throw() {
  string result("comm::handler::ClientSocket { ");
  result += comm::Handler::asString();
  result += " | ";
  result += functions::asString(a_clientSocket);
  return result += " }";
}

xml::Node* comm::handler::ClientSocket::asXML(xml::Node* parent) const
throw() {
  xml::Node* result = parent->createChild("comm.handler.ClientSocket");
  comm::Handler::asAttribute(result);

  if(a_clientSocket)
    a_clientSocket->asXML(result);

  return result;
}


