// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/core/tracing/Logger.hpp>
#include <anna/core/functions.hpp>
#include <anna/core/mt/Guard.hpp>
#include <anna/core/tracing/TraceMethod.hpp>

#include <anna/xml/Node.hpp>
#include <anna/xml/Attribute.hpp>

#include <anna/comm/internal/RemoteConnection.hpp>
#include <anna/comm/Communicator.hpp>
#include <anna/comm/Device.hpp>
#include <anna/comm/Server.hpp>
#include <anna/comm/ClientSocket.hpp>
#include <anna/comm/internal/ConnectionRecover.hpp>
#include <anna/comm/Receiver.hpp>

#include <anna/comm/handler/RemoteConnection.hpp>

using namespace std;
using namespace anna;

//-------------------------------------------------------------------------------------------
// BackTrace:
// Host::createServer -> Server::connect -> Communicator::attach (RemoteConnection*) ->
// comm::handler::RemoteConnection::initialize
//-------------------------------------------------------------------------------------------

/*
   <Guard Id="0x7fbfffeb68" Name="comm::Communicator::multithreadedAccept (ConnectionRecover)" LineNo="635">
     <Method Id="comm::Server::connect" File="comm.Server.cc" Line="149" LineNo="637">
        <Guard Id="0x7fbfffeb68" Name="comm::Communicator from comm::Server::connect" LineNo="639" Mode="Reentrant">
           <Guard Id="0x690f20" Name="comm::Server::connect" LineNo="641">
              <Method Id="comm::handler::RemoteConnection::initialize" File="handler.db/comm.handler.RemoteConnection.cc" Line="54" LineNo="643">
                 <Guard Id="0x690ff0" Name="comm::ClientSocket (connect)" LineNo="645"/>
                 <Method Id="comm::Communicator::eventCreateConnection" File="comm.Communicator.cc" Line="723" LineNo="655">
                    <Method Id="comm::Delivery::recover" File="comm.Delivery.cc" Line="186" LineNo="659">
                       <Guard Id="0x690750" Name="comm::Delivery::recover" LineNo="661">
                          <Method Id="comm::Delivery::unsafe_recover" File="comm.Delivery.cc" Line="196" LineNo="663">
                             <Guard Id="0x690f20" Name="Resource" LineNo="665" Mode="Reentrant"/>
                          </Method>
                       </Guard>
                    </Method>
                 </Method>
              </Method>
              <Guard Id="0x68f378" Name="anna::ThreadManager::createThread" LineNo="681"/>
           </Guard>
        </Guard>
     </Method>
*/
void comm::handler::RemoteConnection::initialize()
throw(RuntimeException) {
  LOGMETHOD(TraceMethod traceMethod("comm::handler::RemoteConnection", "initialize", ANNA_FILE_LOCATION));

  if(a_remoteConnection == NULL) {
    string msg(asString());
    msg += " | No se ha establecido el comm::RemoteConnection";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  if(a_remoteConnection->getClientSocket()  == NULL) {
    string msg(asString());
    msg += " | No se ha establecido el comm::ClientSocket";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  comm::ClientSocket& clientSocket = *a_remoteConnection->getClientSocket();
  comm::Server* server = a_remoteConnection->getServer();

  try {
    if(clientSocket.isConnected() == false)
      clientSocket.connect();

    server->enable();
    a_communicator->eventCreateConnection(server);
    // Notificacion al Receiver (EDU Febrero 2012)
    Receiver* receiver;

    if((receiver = clientSocket.getReceiver()) != NULL)
      receiver->eventCreateConnection(server);

    setfd(clientSocket.getfd());
  } catch(RuntimeException&) {
    a_communicator->getConnectionRecover()->annotateFault(server);
    throw;
  }
}

comm::ClientSocket* comm::handler::RemoteConnection::getClientSocket()
throw() {
  return (a_remoteConnection != NULL) ? a_remoteConnection->getClientSocket() : NULL;
}

/*
 * Se invoca desde Communicator::detach.
 *
 * [Tx] -> Communicator
 *
 *
 * Al caer en el parche 1.11.9 NO se mantena el orden requerido de bloqueo:
       <Method Id="comm::handler::MetaClientSocket::apply" File="handler.db/comm.handler.MetaClientSocket.cc" Line="45" LineNo="419" InitTime="03/12/2009 12:10:30">
         <Guard Id="0x690ff0" Name="comm::ClientSocket from comm::handler::MetaClientSocket::apply" LineNo="421"/>
         <Guard Id="0x7fbfffeb68" Name="comm::Communicator::detach" LineNo="427">
            <Method Id="comm::handler::RemoteConnection::finalize" File="handler.db/comm.handler.RemoteConnection.cc" Line="101" LineNo="431">
               <Guard Id="0x690f20" Name="comm::Server::reset" LineNo="433">
                  <Guard Id="0x690ff0" Name="comm::Socket::close" LineNo="435">
                     <Guard Id="0x685bc0" Name="PN7anna4comm16TransportFactoryE" LineNo="437"/>
                  </Guard>
               </Guard>
               <Method Id="comm::Communicator::eventBreakConnection (server)" File="comm.Communicator.cc" Line="790" LineNo="447">
                  <Method Id="comm::Delivery::fault" File="comm.Delivery.cc" Line="135" LineNo="451">
                     <Guard Id="0x690750" Name="comm::Delivery::fault" LineNo="453">
                        <Method Id="comm::Delivery::unsafe_fault" File="comm.Delivery.cc" Line="149" LineNo="455">
                           <Guard Id="0x690f20" Name="Resource" LineNo="457"/>
                        </Method>
                     </Guard>
                  </Method>
                  <Guard Id="0x7fbfffeb68" Name="comm::Communicator::setStatus" LineNo="469" Mode="Reentrant"/>
               </Method>
            </Method>
         </Guard>
      </Method>
 *
 */
void comm::handler::RemoteConnection::finalize()
throw() {
  if(a_remoteConnection == NULL)
    return;

  LOGMETHOD(TraceMethod traceMethod("comm::handler::RemoteConnection", "finalize", ANNA_FILE_LOCATION));
  comm::Server* server = a_remoteConnection->getServer();
  Guard guard(server, "comm::Server");
  comm::ClientSocket* clientSocket = a_remoteConnection->getClientSocket();

  if(clientSocket != NULL) {
    Receiver* receiver;

    if((receiver = clientSocket->getReceiver()) != NULL)
      receiver->eventBreakConnection(*clientSocket);

    a_communicator->eventBreakConnection(*clientSocket);
  }

  // Cierra el socket y lo libera
  server->reset();
  // Establece la informacin que gestiona el reintento de uso por parte de los servicios de reparto
  server->disable();
  server->setTimeStamp(functions::millisecond());

  if(a_communicator->hasRequestedStop() == false) {
    a_communicator->eventBreakConnection(server);
    a_communicator->getConnectionRecover()->annotateFault(server);
  }

  delete a_remoteConnection;
  a_remoteConnection = NULL;
}

/*
 * Se invoca en ST
 */
void comm::handler::RemoteConnection::clone()
throw(RuntimeException) {
  comm::ClientSocket& clientSocket = *a_remoteConnection->getClientSocket();

  try {
    if(clientSocket.isConnected() == true)
      clientSocket.close();

    clientSocket.connect();
    setfd(clientSocket.getfd());
  } catch(RuntimeException&) {
    a_communicator->getConnectionRecover()->annotateFault(a_remoteConnection->getServer());
    throw;
  }
}

string comm::handler::RemoteConnection::asString() const
throw() {
  string result("comm::handler::RemoteConnection { ");
  result += comm::Handler::asString();
  result += " | ";
  result += functions::asString(a_remoteConnection);
  return result += " }";
}

xml::Node* comm::handler::RemoteConnection::asXML(xml::Node* parent) const
throw() {
  xml::Node* result = parent->createChild("comm.handler.RemoteConnection");
  comm::Handler::asAttribute(result);

  if(a_remoteConnection)
    a_remoteConnection->asXML(result);

  return result;
}


