// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


// Local
#include <anna/diameter/stack/Format.hpp>
#include <anna/diameter/stack/Dictionary.hpp>

#include <anna/core/functions.hpp>
#include <anna/xml/xml.hpp>

//using namespace anna;

//------------------------------------------------------------------------------
//------------------------------------------------------- Format::getBasicType()
//------------------------------------------------------------------------------
anna::diameter::codec::Format::_v anna::diameter::stack::Format::getBasicType(void) const throw(anna::RuntimeException) {
  if(isDerived()) return a_dictionary->getFormat(a_parentName)->getBasicType();

  if(isReserved())
    throw anna::RuntimeException("Develop error: there is no basic format type for reserved codec::Format", ANNA_FILE_LOCATION);

  return (anna::diameter::codec::Format::asEnum(a_name));
}


//------------------------------------------------------------------------------
//------------------------------------------------------ Format::setParentName()
//------------------------------------------------------------------------------
void anna::diameter::stack::Format::setParentName(const std::string & parentName) throw(anna::RuntimeException) {
  const Format * parent = a_dictionary->getFormat(parentName);

  //if (parent && parent->isDerived()) // actually dtd-verified:
  if(parent && !parent->isBasic())  // actually dtd-verified (and 'Any' is not allowed):
    throw anna::RuntimeException("Only basic diameter format allowed for parent type", ANNA_FILE_LOCATION);

  a_parentName = parentName;
}


//------------------------------------------------------------------------------
//----------------------------------------------------------- Format::asString()
//------------------------------------------------------------------------------
std::string anna::diameter::stack::Format::asString(void) const throw() {
  std::string trace;
  //trace = "Format '";
  trace = "'";
  trace += a_name;
  trace += "'";
  const Format * parent = a_dictionary->getFormat(a_parentName);

  if(parent) {
    trace += ", derived from ";
    trace += parent->asString();
  }

  return (trace);
}


//------------------------------------------------------------------------------
//-------------------------------------------------------------- Format::asXML()
//------------------------------------------------------------------------------
anna::xml::Node* anna::diameter::stack::Format::asXML(anna::xml::Node* parent) const throw() {
//   <!ELEMENT format EMPTY>
//   <!ATTLIST format name CDATA #REQUIRED parent-type ( OctetString | Integer32 | Integer64 | Unsigned32 | Unsigned64 | Float32 | Float64 ) #REQUIRED>
  anna::xml::Node* result = parent->createChild("format");
  result->createAttribute("name", a_name);
  result->createAttribute("parent-type", a_parentName);
  return result;
}

