// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <stdlib.h>

#include <anna/core/functions.hpp>

#include <anna/http/Header.hpp>
#include <anna/http/internal/Token.hpp>

using namespace std;
using namespace anna;

const char* http::Header::st_names [http::Header::Type::End] = {
  "None",
  "Cache-Control", "Connection", "Date", "Pragma", "Trailer", "Transfer-Encoding", "Upgrade", "Via",
  "Warning",
  "Accept", "Accept-Charset", "Accept-Encoding", "Accept-Language", "Authorization", "Expect", "From",
  "Host", "If-Match", "If-Modified-Since", "If-None-Match", "If-Range", "If-Unmodified-Since", "Max-Forwards",
  "Proxy-Authorization", "Range", "Referer", "TE", "User-Agent",
  "Allow", "Content-Encoding", "Content-Language", "Content-Length", "Content-Location", "Content-MD5",
  "Content-Range", "Content-Type", "Expires", "Last-Modified",
  "Accept-Ranges", "Age", "ETAG", "Location", "Proxy-Authenticate", "Retry-After", "Server", "Vary",
  "WWW-Authenticate", "Unknown"
};

http::Header* http::Header::initialize(const Type::_v type)
throw(RuntimeException) {
  if(type == Type::None || type >= Type::Unknown) {
    string msg(asString());
    msg += " | Type: ";
    msg += functions::asString((int) type);
    msg += " | Invalid header initializer";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  a_type = type;

  if(type >= Type::CacheControl && type <= Type::Warning)
    a_category = Category::General;
  else if(type >= Type::Accept && type <= Type::UserAgent)
    a_category = Category::Request;
  else if(type >= Type::Allow && type <= Type::LastModified)
    a_category = Category::Entity;
  else if(type >= Type::AcceptRanges && type <= Type::WWWAuthenticate)
    a_category = Category::Response;
  else
    a_category = Category::Extension;

  delete a_extensionName;
  a_extensionName = NULL;
  return this;
}

http::Header* http::Header::initialize(const string& name)
throw(RuntimeException) {
  if(a_extensionName == NULL)
    a_extensionName = new string(name);
  else
    *a_extensionName = name;

  a_type = Type::Unknown;
  a_category = Category::Extension;
  return this;
}

const int http::Header::getIntegerValue() const
throw() {
  return atoi(a_value.c_str());
}

void http::Header::setValue(const http::Token* token)
throw() {
  if(token == NULL)
    a_value.clear();
  else
    a_value = token->getStringValue();
}

void http::Header::setValue(const int value)
throw() {
  a_value = anna::functions::asString(value);
}

http::Header& http::Header::operator = (const Header & other)
throw() {
  if(this == &other)
    return *this;

  a_type = other.a_type;
  a_value = other.a_value;

  if(other.a_extensionName == NULL) {
    delete a_extensionName;
    a_extensionName = NULL;
  } else {
    if(a_extensionName == NULL)
      a_extensionName = new string(*other.a_extensionName);
    else
      *a_extensionName = *other.a_extensionName;
  }

  return *this;
}

int http::Header::compare(const char* str, const int flags) const
throw() {
  const char* p = a_value.c_str();
  char* dup(NULL);
  int result;

  if(flags & Compare::LeftTrim) {
    while(*p && *p == ' ')
      p ++;
  }

  if((flags & Compare::RightTrim) && *p != 0 && anna_strchr(p, ' ') != NULL) {
    char* ww;
    ww = dup = strdup(p);
    p = const_cast <const char*>(dup);
    ww += anna_strlen(ww) - 1;

    while(ww >= dup && *ww == ' ')
      ww --;

    *(ww + 1) = 0;
  }

  result = (flags & Compare::NoCase) ? strcasecmp(p, str) : anna_strcmp(p, str);

  if(dup != NULL)
    free(dup);

  return result;
}

std::string http::Header::asString() const
throw() {
  string result("http::Header { Type: ");
  result += asLiteral(a_type);

  if(a_extensionName != NULL) {
    result += " | Name: ";
    result += *a_extensionName;
  }

  result += " | Value: ";

  if(a_value.empty())
    result += "<null>";
  else
    result += a_value;

  return result += " }";
}

string http::Header::code() const
throw() {
  string result;

  if(a_category == Category::Extension)
    result = (a_extensionName == NULL) ? "None" : a_extensionName->c_str();
  else
    result = asLiteral(a_type);

  if(a_value.length() > 0) {
    result += ':';
    result += a_value;
  }

  return result;
}

http::Header::Type::_v http::Header::asType(const http::Token* token)
throw() {
  for(int i = Type::Begin; token != NULL && i != Type::End; i ++) {
    if(token->match(st_names [i]) == true)
      return (Type::_v) i;
  }

  return Type::None;
}

const char* http::Header::asLiteral(const http::Header::Type::_v type)
throw() {
  return (type < Type::End) ? st_names [type] : "Extension";
}

